"""hook to normalize tag names

:organization: Logilab
:copyright: 2003-2010 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
:contact: http://www.logilab.fr/ -- mailto:contact@logilab.fr
"""
__docformat__ = "restructuredtext en"

from cubicweb.server.hook import Hook
from cubicweb.predicates import is_instance


class AddUpdateTagHook(Hook):
    """ensure tag names are lower case and commas are forbidden"""
    __regid__ = 'add_update_tag'
    __select__ = Hook.__select__ & is_instance('Tag')
    events = ('before_add_entity', 'before_update_entity',)

    def __call__(self):
        if 'name' in self.entity.cw_edited:
            name = self.entity.cw_attr_cache['name'].lower()
            name = ' - '.join(part.strip() for part in name.split(','))
            self.entity.cw_edited['name'] = name



from cubicweb.devtools.testlib import CubicWebTC


class HooksTC(CubicWebTC):

    def setup_database(self):
        with self.admin_access.repo_cnx() as cnx:
            cnx.create_entity('BlogEntry', title=u"une news !",
                              content=u"cubicweb c'est beau")
            cnx.commit()

    def test_tag_lower_cased(self):
        with self.admin_access.client_cnx() as cnx:
            teid = cnx.execute('INSERT Tag T: T name "ToTo", T tags X '
                               'WHERE X is BlogEntry')[0][0]
            tname = cnx.execute('Any TN WHERE T eid %(t)s, T name TN',
                                {'t': teid})[0][0]
            self.assertEqual(tname, 'toto')

            teid = cnx.execute('SET T name "TuTu" WHERE T is Tag')[0][0]
            tname = cnx.execute('Any TN WHERE T eid %(t)s, T name TN',
                                {'t': teid})[0][0]
            self.assertEqual(tname, 'tutu')

    def test_tag_lower_cased_and_no_comma(self):
        with self.admin_access.client_cnx() as cnx:
            teid = cnx.execute('INSERT Tag T: T name "ToTo, Titi", T tags X '
                               'WHERE X is BlogEntry')[0][0]
            tname = cnx.execute('Any TN WHERE T eid %(t)s, T name TN',
                                {'t': teid})[0][0]
            self.assertEqual(tname, 'toto - titi')


if __name__ == '__main__':
    import unittest
    unittest.main()



"""entity class for Tag entities

:organization: Logilab
:copyright: 2003-2010 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
:contact: http://www.logilab.fr/ -- mailto:contact@logilab.fr
"""
__docformat__ = "restructuredtext en"

from cubicweb.entities import AnyEntity, fetch_config


class Tag(AnyEntity):
    """customized class for Tag entities"""
    __regid__ = 'Tag'
    fetch_attrs, cw_fetch_order = fetch_config(['name'])

    def closest_tags_rset(self):
        return self._cw.execute('Any CT, COUNT(X) GROUPBY CT ORDERBY 2 DESC '
                                'LIMIT 5 '
                                'WHERE T tags X, T eid %(x)s, CT tags X, '
                                'NOT CT eid %(x)s', {'x': self.eid})
